import java.awt.*;
import javax.swing.*;
import java.io.Serializable;
import java.math.*;
import java.io.PrintStream;
import java.util.ArrayList;

public class CircleLayout implements LayoutManager2, Serializable
{
  int xcenter;
  int ycenter;
  int raio;
  Dimension salvo = null;
  ArrayList m_list = new ArrayList(10);
  int m_x[] = null;
  int m_y[] = null;
  int nChildren = 0;
  Component m_centro;
  public static final Integer CENTRO = new Integer(0);
  public static final Integer BORDA = new Integer(1);

  public CircleLayout(Container target)
  {
	this.target = target;
  }  

  CircleLayout(Container target, PrintStream dbg) {
	this(target);
	this.dbg = dbg;
  }  

  /**
   * Indicates that a child has changed its layout related information,
   * and thus any cached calculations should be flushed.
   *
   * @param target  the affected container
   *
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   */
  public void invalidateLayout(Container target) {
	checkContainer(target);
  }  

  /**
   * Not used by this class.
   *
   * @param name the name of the component
   * @param comp the component
   */
  public void addLayoutComponent(String name, Component comp) {
	addLayoutComponent(comp,BORDA);
  }  

  /**
   * Not used by this class.
   *
   * @param comp the component
   */
  public void removeLayoutComponent(Component comp)
  {
	m_list.remove(comp);
  }  

  /**
   * Not used by this class.
   *
   * @param comp the component
   * @param constraints constraints
   */
  public void addLayoutComponent(Component comp, Object constraints)
  {
	if (constraints == CENTRO)
	  m_centro = comp;
	else
	  m_list.add(comp);
  }  

  /**
   * Returns the preferred dimensions for this layout, given the components
   * in the specified target container.
   *
   * @param target  the container that needs to be laid out
   * @return the dimensions >= 0 && <= Integer.MAX_VALUE
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   * @see Container
   * @see #minimumLayoutSize
   * @see #maximumLayoutSize
   */
  public Dimension preferredLayoutSize(Container target) {
	checkContainer(target);

	int mraio = minimumRaio()-5;
	int dim = mraio*2;
	Dimension size = new Dimension(dim,dim);
	Insets insets = target.getInsets();
	//System.out.println("Insets: "+insets.toString());
	size.width = (int) Math.min((long) size.width + (long) insets.left + (long) insets.right, Integer.MAX_VALUE);
	size.height = (int) Math.min((long) size.height + (long) insets.top + (long) insets.bottom, Integer.MAX_VALUE);
	return size;
  }  

  /**
   * Returns the minimum dimensions needed to lay out the components
   * contained in the specified target container.
   *
   * @param target  the container that needs to be laid out
   * @return the dimensions >= 0 && <= Integer.MAX_VALUE
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   * @see #preferredLayoutSize
   * @see #maximumLayoutSize
   */
  public Dimension minimumLayoutSize(Container target) {
	checkContainer(target);
	int mraio = minimumRaio();
	int dim = mraio*2;
	Dimension size = new Dimension(dim,dim);
	Insets insets = target.getInsets();
	//System.out.println("Insets: "+insets.toString());
	size.width = (int) Math.min((long) size.width + (long) insets.left + (long) insets.right, Integer.MAX_VALUE);
	size.height = (int) Math.min((long) size.height + (long) insets.top + (long) insets.bottom, Integer.MAX_VALUE);
	return size;
  }  

  int minimumDistance()
  {
	int nchild = m_list.size();
	if (nchild == 0) return 0;
	int minDist = 0;
	JComponent cant = (JComponent)m_list.get(0);
	int want = cant.getPreferredSize().width;
	int hant = cant.getPreferredSize().height;
	for (int i = 1; i < nchild; i++)
	{
	  JComponent c = (JComponent)m_list.get(i);
	  int w = c.getPreferredSize().width;
	  int h = c.getPreferredSize().height;
	  int x = (w+want)/2;
	  int y = (h+want)/2;
	  int d = (int)Math.sqrt(x*x+y*y);
	  if (minDist < d) minDist = d;
	  want = w;
	  hant = h;
	}

	return minDist;
  }  

  int minimumRaio()
  {
	int nchild = m_list.size();
	int minDist = minimumDistance();
	double grau = 2*Math.PI/nchild;
	int raio = 100;
	int d = 0;
	do
	{
	  raio += 50;
	  int xp0 = raio;
	  int yp0 = 0;
	  int xp1 = (int)(raio*Math.cos(grau));
	  int yp1 = (int)(raio*Math.sin(grau));
	  int dx = xp1-xp0;
	  int dy = yp1-yp0;
	  d = (int)Math.sqrt(dx*dx+dy*dy);
	  //System.out.println("Distancia em calculo = "+d+" raio = "+raio);
	} while (d < minDist);
	return raio;
  }  
  /**
   * Returns the maximum dimensions the target container can use
   * to lay out the components it contains.
   *
   * @param target  the container that needs to be laid out
   * @return the dimenions >= 0 && <= Integer.MAX_VALUE
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   * @see #preferredLayoutSize
   * @see #minimumLayoutSize
   */
  public Dimension maximumLayoutSize(Container target) {
	checkContainer(target);

	Dimension size = new Dimension(1000,1000);
	Insets insets = target.getInsets();
	size.width = (int) Math.min((long) size.width + (long) insets.left + (long) insets.right, Integer.MAX_VALUE);
	size.height = (int) Math.min((long) size.height + (long) insets.top + (long) insets.bottom, Integer.MAX_VALUE);
	return size;
  }  

  /**
   * Returns the alignment along the X axis for the container.
   * If the box is horizontal, the default
   * alignment will be returned. Otherwise, the alignment needed
   * to place the children along the X axis will be returned.
   *
   * @param target  the container
   * @return the alignment >= 0.0f && <= 1.0f
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   */
  public float getLayoutAlignmentX(Container target)
  {
	checkContainer(target);
	return 0.5f;
  }  

  /**
   * Returns the alignment along the Y axis for the container.
   * If the box is vertical, the default
   * alignment will be returned. Otherwise, the alignment needed
   * to place the children along the Y axis will be returned.
   *
   * @param target  the container
   * @return the alignment >= 0.0f && <= 1.0f
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   */
  public float getLayoutAlignmentY(Container target)
  {
	checkContainer(target);
	return 0.5f;
  }  

  /**
   * Called by the AWT <!-- XXX CHECK! --> when the specified container
   * needs to be laid out.
   *
   * @param target  the container to lay out
   *
   * @exception AWTError  if the target isn't the container specified to the
   *                      CircleLayout constructor
   */
  public void layoutContainer(Container target)
  {
	checkContainer(target);
	calculaPosicoes();
	double grau = 2*Math.PI/nChildren;
	for (int i = 0; i < nChildren; i++)
	{
	  int x = m_x[i];
	  int y = m_y[i];
	  JComponent c = (JComponent)m_list.get((int)i);
	  int w = c.getPreferredSize().width;
	  int h = c.getPreferredSize().height;
	  x -= w/2;
	  y -= h/2;
	  c.setBounds(x+xcenter,y+ycenter,w,h);
	  //System.out.println("Componente "+i+" x = "+x+" y = "+y);
	}
	if (m_centro != null)
	{
	  int w = m_centro.getPreferredSize().width;
	  int h = m_centro.getPreferredSize().height;
	  m_centro.setBounds(xcenter-w/2,ycenter-h/2,w,h);
	}
  }  

  void checkContainer(Container target) {
	if (this.target != target) {
	  throw new AWTError("CircleLayout can't be shared");
	}
  }  

  void calculaPosicoes()
  {
	calculaRaio();
	double grau = 2*Math.PI/nChildren;
	m_x = new int[nChildren];
	m_y = new int[nChildren];
	for (int i = 0; i < nChildren; i++)
	{
	  int x = (int)(raio*Math.cos(((double)i)*grau));
	  int y = (int)(raio*Math.sin(((double)i)*grau));
	  m_x[i] = x;
	  m_y[i] = y;
	}
	/*
	 for (int i = 1; i < nChildren; i++)
	 {
	 int dx = m_x[i] - m_x[i-1];
	 int dy = m_y[i] - m_y[i-1];
	 int dist = (int)Math.sqrt((dx*dx)+(dy*dy));
	 System.out.println("Pos "+i+", distancia = "+dist);
	 }
	 */
  }  

  void calculaRaio()
  {
	Dimension s = target.getSize();
	if (s == salvo)
	{
	  return;
	}
	salvo = s;
	//Dimension min = minimumLayoutSize(target);
	//System.out.println(s.toString());
	//System.out.println(min.toString());
	Insets insets = target.getInsets();
	//System.out.println("Insets: "+insets.toString());
	ycenter = (s.height - insets.top - insets.bottom)/2;
	xcenter = (s.width - insets.left - insets.right)/2;
	raio = Math.min(ycenter,xcenter);
	ycenter += insets.top;
	xcenter += insets.left;
	//System.out.println("Center = "+xcenter+","+ycenter);
	int maxxy = 0;
	nChildren = m_list.size();
	for (double i = 0; i < nChildren; i+= 1.0)
	{
	  JComponent c = (JComponent)m_list.get((int)i);
	  int w = c.getPreferredSize().width;
	  int h = c.getPreferredSize().height;
	  if (w > maxxy) maxxy = w;
	  if (h > maxxy) maxxy = h;
	}
	raio -= maxxy/2 ;
  }  

  private Container target;

  private transient PrintStream dbg;
}