/*
* User: eoliveira
* Date: 03/07/2002
* Time: 19:25:36
*/

import MonitoredEntities.*;
import EventEvaluator.Resource.ResourceEventRepository;
import EventEvaluator.Resource.ResourceEventRepositoryHelper;
import EventEvaluator.Resource.EntityParameterRange;
import EventEvaluator.Interation.InterationEventRepository;
import EventEvaluator.Interation.InterationEventRepositoryHelper;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CosNaming.NameComponent;
import Utilities.repClient.EntityTypeClient;
import Utilities.repClient.EntityClient;
import Utilities.repClient.ParameterClient;
import Utilities.repClient.ResourceEventClient;
import Utilities.repClient.InterationEventClient;

import javax.swing.*;
import java.util.Vector;
import java.util.Enumeration;

public class EntityRepositoryConnector
{
  private static EntityRepository eREP;
  private static ResourceEventRepository reREP;
  private static InterationEventRepository ieREP;
  private static org.omg.CORBA.ORB orb;
  private static EventEvaluator.Resource.ResourceManager rManager;

  private static void inicializa()
  {
    try {
      // get reference to the initial naming context
      NamingContextExt nc =
              NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));

      // get reference to entities repository
      NameComponent [] name = new NameComponent[2];
      name[0] = new NameComponent("Adaptacao","context");
      name[1] = new NameComponent("EntityRepository","service");
      eREP = EntityRepositoryHelper.narrow(nc.resolve(name));

      // get reference to ResourceEvent objects repository
      NameComponent [] name2 = new NameComponent[2];
      name2[0] = new NameComponent("Adaptacao","context");
      name2[1] = new NameComponent("ResourceEventRepository","service");
      reREP = ResourceEventRepositoryHelper.narrow(nc.resolve(name2));

      // get reference to InterationEvent objects repository
      NameComponent [] name3 = new NameComponent[2];
      name3[0] = new NameComponent("Adaptacao","context");
      name3[1] = new NameComponent("InterationEventRepository","service");
      ieREP = InterationEventRepositoryHelper.narrow(nc.resolve(name3));

      // get reference to resource event evaluator
      NameComponent [] rmname = new NameComponent[2];
      rmname[0] = new NameComponent("Adaptacao","context");
      rmname[1] = new NameComponent("ResourceManager","service");
      System.out.println ("obtaining reference to resource manager");
      rManager =
              EventEvaluator.Resource.ResourceManagerHelper.narrow(nc.resolve(rmname));
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),e);
      e.printStackTrace();
      System.out.println("givin up...");
    }
  }

  public EntityRepositoryConnector()
  {
    // initialize
    //3orb  = org.omg.CORBA.ORB.init(args, null);
    java.util.Properties props = new java.util.Properties();
    props.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
    props.put("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");
    props.put("jacorb.poa.monitoring","off");
    props.put("ORBInitRef.NameService","http://easter3/~eider/NS_Ref");
    //props.put("ORBInitRef.NameService","http://localhost/mestrado/NS_Ref");
    props.put("ORBInitRef.TradingService","http://www.x.y.z/~user/TraderRef");
    String[] args = {};

    orb = org.omg.CORBA.ORB.init(args, props);
    inicializa();
  }

  String getXMLEntities()
  {
    String []meids = eREP.list_me();
    String xml =  "  <Entities>";
    for (int i = 0; i < meids.length; i++)
    {
      xml += "    <Entity meid=\"" + meids[i]+"\" ";
      try {
        Entity e = eREP.get_me_ref(meids[i]);
        xml += "Description = \""+e.description()+"\" ";
        EntityType et = e.metype();
        xml += "Metid = \""+et.metid()+"\" ";
        int nattr = e.num_attributes();
        String[] attr = new String[nattr];
        attr = e.attributes();
        String sattr = "";
        for (int j = 0; j < nattr; j++)
          sattr += attr[j]+ " ";
        xml += "Attributes = \""+sattr+"\" ";

        xml += "Range_Atual = \"&lt;table border=0&gt;&lt;tr&gt;&lt;th&gt;Parameter&lt;/th";
        xml += "&gt;&lt;th&gt;Value&lt;/th&gt;&lt;/tr&gt;";
        EntityParameterRange[] pr = rManager.list_me_parameter_range(meids[i]);
        for (int l = 0; l < pr.length; l++)
        {
          xml += "&lt;tr&gt;&lt;td&gt;&lt;b&gt;&lt;font color=blue&gt;"
                  + pr[l].pid
                  + "&lt;/font&gt;&lt;/b&gt;&lt;/td&gt;&lt;td&gt;&lt;font color=red&gt;"
                  + pr[l].range
                  + "&lt;/font&gt;&lt;/td&gt;&lt;/tr&gt;\n";
        }
        xml += "&lt;/table&gt;\" ";
      }
      catch (NoSuchEntity nse) {
        System.out.println("entity "+meids[i]+" does not exist !");
        JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nse,
                                      "entity "+meids[i]+" does not exist !",JOptionPane.ERROR_MESSAGE);
      }
      xml += " />";
    }
    xml +=  "  </Entities>";
    return xml;
  }

  String getXMLEntityTypes()
  {
    String []metids = eREP.list_metype();
    String xml = "  <EntityTypes>";
    for (int i = 0; i < metids.length; i++)
    {

      xml += "    <EntityType metid=\"" + metids[i]+"\" ";
      try {
        EntityType et = eREP.get_metype_ref(metids[i]);
        xml += "Description = \""+et.description()+"\" ";
      }
      catch (NoSuchEntityType nset) {
        System.out.println("entity type "+metids[i]+" does not exist !");
        JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nset,
                                      "entity type "+metids[i]+" does not exist !",JOptionPane.ERROR_MESSAGE);
      }
      xml += " />";
    }
    xml +=  "  </EntityTypes>";
    return xml;
  }

  String getXMLParameters()
  {
    String []pids = eREP.list_parameter();
    String xml =   "  <Parameters>";
    for (int i = 0; i < pids.length; i++)
    {
      xml += "    <Parameter pid=\"" + pids[i]+"\" ";
      try {
        Parameter p = eREP.get_parameter_ref(pids[i]);
        EntityType et = p.metype();
        xml += "Description = \""+p.description()+"\" ";
        xml += "Metid = \""+et.metid()+"\" ";
        int nr = p.num_ranges();
        range[] rangeList = new range[nr];
        rangeList = p.ranges();
        for (int j = 0; j < nr; j++) {
          xml += "Range_"+j+" = \""+rangeList[j].initial_value+" -> "+rangeList[j].final_value+"\" ";
        }
        xml += " />";
      }
      catch (NoSuchParameter nsp) {
        System.out.println("parameter "+pids[i]+" does not exist !");
        JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nsp,
                                      "parameter "+pids[i]+" does not exist !",JOptionPane.ERROR_MESSAGE);
      }
    }
    xml +=  "  </Parameters>";
    return xml;
  }

  public String getXMLRepository()
  {
    String xml =  "<EntityRepository>";
    xml += getXMLEntityTypes();
    xml += getXMLEntities();
    xml += getXMLParameters();
    xml +=  "</EntityRepository>";
    return xml;
  }

  public void addParameter(String pid, String description, String metid, Vector vranges)
  {
    try {
      EntityType et = eREP.get_metype_ref(metid);
      range[] rangeList = new range[vranges.size()];
      int i =0;
      for (Enumeration e = vranges.elements() ; e.hasMoreElements() ;) {
        rangeList[i] = (range)e.nextElement();
        i++;
      }
      eREP.add_parameter(pid, description, et, rangeList);
      System.out.println("added parameter "+pid+" !");
    }
    catch (NoSuchEntityType nset) {
      System.out.println("entity type"+metid+" does not exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nset,
                                    "entity type "+metid+" does not exist !",JOptionPane.ERROR_MESSAGE);
    }
    catch (ParameterAlreadyExist pae) {
      System.out.println("parameter "+pid+" already exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),pae,
                                    "parameter "+pid+" already exist !",JOptionPane.ERROR_MESSAGE);
    }
  }
  public void addEntity(String meid, String description, String metid,Vector vattr)
  {
    try {
      EntityType et = eREP.get_metype_ref(metid);
      String[] attr = new String[vattr.size()];
      int i=0;
      for (Enumeration e = vattr.elements() ; e.hasMoreElements() ;) {
        attr[i] = (String)e.nextElement();
        i++;
      }
      eREP.add_me(meid, et, description, attr);
      System.out.println("added entity "+meid+" !");
    }
    catch (NoSuchEntityType nset) {
      System.out.println("entity type"+metid+" does not exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nset,
                                    "entity type "+metid+" does not exist !",JOptionPane.ERROR_MESSAGE);
    }
    catch (EntityAlreadyExist eae) {
      System.out.println("entity "+meid+" already exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),eae,
                                    "entity "+meid+" already exist !",JOptionPane.ERROR_MESSAGE);
    }
  }

  public void addEntityType(String metid, String description)
  {
    try {
      eREP.add_metype(metid, description);
      System.out.println("added entity type "+metid+" !");
    }
    catch (EntityTypeAlreadyExist etae) {
      System.out.println("entity type"+metid+" already exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),etae,
                                    "entity type"+metid+" already exist !",JOptionPane.ERROR_MESSAGE);
    }
  }

  public void removeEntityType(String metid)
  {
    try {
      eREP.erase_metype(metid);
      System.out.println("removed entity type "+metid+" !");
    }
    catch (NoSuchEntityType nset) {
      System.out.println("entity type "+metid+" don't exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nset,
                                    "entity type "+metid+" don't exist !",JOptionPane.ERROR_MESSAGE);
    }
    catch (RIViolate riv) {
	  System.out.println("Relational Integrity Violation ! "+metid+" not deleted !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),riv,
                                    "Relational Integrity Violation ! "+metid+" not deleted !",
                                    JOptionPane.ERROR_MESSAGE);
	}
  }
  public void removeParameter(String pid)
  {
    try {
      eREP.erase_parameter(pid);
      System.out.println("removed parameter "+pid+" !");
    }
    catch (NoSuchParameter nsp) {
      System.out.println("paramter "+pid+" don't exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nsp,
                                    "parameter "+pid+" don't exist !",JOptionPane.ERROR_MESSAGE);
    }
  }
  public void removeEntity(String meid)
  {
    try {
      eREP.erase_me(meid);
      System.out.println("removed entity "+meid+" !");
    }
    catch (NoSuchEntity nse) {
      System.out.println("entity "+meid+" don't exist !");
      JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),nse,
                                    "entity "+meid+" don't exist !",JOptionPane.ERROR_MESSAGE);
    }
  }
}

