/*
 * ActiveEvent
 * Version 1.0
 */

package EventEvaluator.Interation;

import java.lang.reflect.*;
import java.util.*;
import java.sql.*;
import java.io.*;
import java.lang.*;

/**
 * Event that is beeing evaluated by the Interation Manager
 * 
 * @author Francisco Silva
 * @version 1.0
 */
public class ActiveEvent {

    private String eid;
    private Notifier notifier;
    private InterationEvent ie;
    private String description;
    private int frequency;
    private String class_name;
    private boolean is_suspended;
    private EventEvaluator.Interation.Evaluators.Evaluator eobj;
    private Frequency fobj;
    private Suspend   sobj;

    /**
     * Obtains data from interation event <code>ie</code>.
     * @param ie reference to interation event related to this active event
     */
    public ActiveEvent(InterationEvent pie, Notifier pnotifier) 
	throws EventEvaluator.EventClassNotFound, NoSuchMethodException {
	this.ie = pie;
	this.notifier = pnotifier;
	this.eid = ie.eid();
	this.description = ie.description();
	this.frequency = ie.frequency();
	this.class_name = ie.class_name();
	
	// instanciates auxiliary objects
	this.is_suspended = false;
	this.fobj = new Frequency(frequency);
	this.sobj = new Suspend();

	Class evaluatorDefinition;
	Class[] theArgsClass = new Class[] {
	    EventEvaluator.Interation.Notifier.class,
	    EventEvaluator.Interation.Frequency.class, 
	    EventEvaluator.Interation.Suspend.class};
	Object[] theArgs = new Object[] {notifier, fobj, sobj};
	Constructor theArgsConstructor;
	
	// instanciates the evaluator     
	try {
	    evaluatorDefinition = Class.forName(class_name);
	    theArgsConstructor = 
		evaluatorDefinition.getConstructor(theArgsClass);
	    eobj = (EventEvaluator.Interation.Evaluators.Evaluator) createObject(theArgsConstructor, theArgs);
	    if (eobj == null) {
		System.out.println("ActiveEvent Constructor: createObject returned null !");
	    }
	    else {
		System.out.println("Starting Evaluator");
		eobj.start();
	    }
	} 
	catch (ClassNotFoundException cnfe) {
	    throw new EventEvaluator.EventClassNotFound(eid, class_name);
	} 
	catch (NoSuchMethodException nsme) {
	    System.out.println("ActiveEvent.Constructor: NoSuchMethodException !");
	    throw nsme;
	}  	
	
    }
 
    /**
     * Returns the event identification.
     * @return a string containing the event identification
     */
    public String eid() {
	return eid;
    }

    public String description() {
	return description;
    }

    public String class_name() {
	return class_name;
    }

    /**
     * Returns the frequency of this event.
     * @return the frequency (in seconds)
     */
    public int frequency() {
	return frequency;
    }

    /**
     * Returns if the evaluation of this active event is supended or not.
     * @return <code>TRUE</code> if the evaluation of this event
     * is suspended, <code>FALSE</code> otherwise
     */
    public boolean is_suspended() {
	return is_suspended;
    }

    /**
     * Suspend the evaluation of this active event.
     */
    public void suspend() {
	is_suspended = true;
	sobj.suspend();
    }
  
    /**
     * Resume the evaluation of this active event.
     */  
    public void resume() {
	is_suspended = false;
	sobj.resume();
    }

    public void change_frequency (int new_frequency) {
	frequency = new_frequency;
	fobj.set(new_frequency);
    }

    private static Object createObject(Constructor constructor, 
				       Object[] arguments) {
	
	// System.out.println ("Constructor: " + constructor.toString());
	Object object = null;
	
	try {
	    object = constructor.newInstance(arguments);
	    // System.out.println ("Object: " + object.toString());
	    return object;
	} catch (InstantiationException e) {
	    System.out.println(e);
	} catch (IllegalAccessException e) {
	    System.out.println(e);
	} catch (IllegalArgumentException e) {
	    System.out.println(e);
	} catch (InvocationTargetException e) {
	    System.out.println(e);
	}
	return object;
    }

} // end class ActiveEvent



