/*
 * InterationEventRepositoryImpl
 * Version 1.0
 */
package EventEvaluator.Interation;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;

/** 
 * Implements the EventEvaluator.Interation.InterationEventRepository interface.
 * 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class InterationEventRepositoryImpl extends InterationEventRepositoryPOA {

    private org.omg.CORBA.ORB orb;
    private POA the_poa; 
    private Persistence pobj;
    private InterationEventLocatorImpl ie_locator;
    
    /**
     * Receive references to orb, poa and <code>Persistence</code> objects.
     * @param orb reference to the orb
     * @param the_poa refence to the poa
     * @param Pobj reference to Persistence object
     * @param ie_locator reference to interation event locator
     */
    public InterationEventRepositoryImpl(org.omg.CORBA.ORB orb, POA the_poa, 
					 Persistence pobj, 
					 InterationEventLocatorImpl ie_locator)
    {
	this.orb = orb;
	this.the_poa = the_poa;
	this.pobj = pobj;
	this.ie_locator = ie_locator;
    }

    /** 
     * Creates a interation event object.
     * Creates a interation event object and saves it state to persistent storage.
     * @param eid the identification of the object being created
     * @param description its description 
     * @param frequency 
     * @param class_name
     * @return the ior reference for the interation event object that was created
     */
    public InterationEvent store_interation_event(String eid, String description, 
						  int frequency, String class_name) 
      throws EventEvaluator.EventAlreadyExist,
	     EventEvaluator.EventClassNotFound {
	if (pobj.IEExist(eid))
	  throw new EventEvaluator.EventAlreadyExist(eid); 
	// System.out.println("criando evento de interacao " + eid);
	InterationEvent ie = null;
	try
	    {
	      check_class_name(eid, class_name);
	      org.omg.CORBA.Object obj = the_poa.create_reference_with_id
		  (eid.getBytes(), InterationEventHelper.id());
	      ie = InterationEventHelper.narrow(obj);
	    }
	catch (org.omg.PortableServer.POAPackage.WrongPolicy wp)
	    {
		wp.printStackTrace();
	    }
	catch (EventEvaluator.EventClassNotFound icnf)
	    {
	      throw icnf;
	    }
	catch (Exception exc) {
	    System.out.println("Exception on method store_interation_event()");
	}
	String strIOR = orb.object_to_string(ie);

	// save object state
	pobj.IESaveState (eid, strIOR, description, frequency, class_name);
	System.out.println("Evento de Interacao " + eid + " criado...");
	// System.out.println(strIOR);
	return ie;
    }

    /**
     * Destroy an interation event object by removing it from persistent storage.
     * @param eid the identification of the interation event to be destroyed
     */
    public void erase(String eid) throws EventEvaluator.NoSuchEvent {
	System.out.println("tentando excluir evento de interacao " + eid + "...");
	if (! pobj.IEExist(eid))
	    throw new EventEvaluator.NoSuchEvent(eid);
	pobj.IEDeleteState (eid);
	ie_locator.free(eid);
	System.out.println("Evento de Interacao " + eid + " excluido...");
    }

    /**
     * Returns the ior reference for the interation event identified by <code>eid</code>.
     * @param eid the identification of the interation event
     * @return the reference for the interation event
     */
    public InterationEvent get_ie_ref(String eid) throws EventEvaluator.NoSuchEvent {
	if (! pobj.IEExist(eid))
	    throw new EventEvaluator.NoSuchEvent(eid); 
	String strIOR = pobj.IEGetRef(eid);
	InterationEvent ie =
	    InterationEventHelper.narrow(orb.string_to_object(strIOR));
	System.out.println("retornando referencia a " + eid + "...");
	// System.out.println(strIOR);
	return ie;
    }

  private void check_class_name(String eid, String class_name) 
      throws EventEvaluator.EventClassNotFound {
      Object object = null;
      try {
          Class classDefinition = Class.forName(class_name);
      }
      catch (ClassNotFoundException e) {
          throw new EventEvaluator.EventClassNotFound(eid, class_name);
      }
  }
 
} // end class InterationEventRepositoryImpl
  





