/*
 * ServerInterationManager
 * Version 1.0
 */
package EventEvaluator.Interation;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosTransactions.*;
import org.omg.CosNaming.*;

import Utilities.Log;

import java.io.*;

/** 
 * Implements the server process that contains the interation event evaluator.
 * @author Francisco Silva
 * @version 1.0
 */
public class ServerInterationManager {

    public static Utilities.Log logObj;

    /**
     * Creates the interation event evaluator servant and register it with the
     * name service.
     */
    public static void main(String[] args) {
    	
	if (args.length != 0 && args.length != 1) {
	    System.out.println ("Use: jaco EventEvaluator.Interation.ServerInterationManager [-p]");
	    System.exit(1);
	}
	if ((args.length == 1) && (args[0].equals("-p"))) {
	    logObj = logObj.Instance();
	    logObj.createFile(System.getProperty("user.home")+"/temp/tempo/verificaEventoInteracao.txt");	
	    logObj.setApplicationName("InterationEventEvaluator");
	}

	try {
	    // obtain reference to orb
	    org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);

	    // obtain reference to root poa
	    POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	    
	    // obtain reference to name service
	    NamingContextExt rootContext = 
		NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));
    
	    // obtain reference to InterationEvent repository
	    NameComponent [] name_ie_r = new NameComponent[2];
	    name_ie_r[0] = new NameComponent("Adaptacao","context");
	    name_ie_r[1] = new NameComponent("InterationEventRepository","service");
	    System.out.println ("obtendo referencia ao repositorio de eventos de recurso");   
	    InterationEventRepository ie_r = 
		InterationEventRepositoryHelper.narrow(rootContext.resolve(name_ie_r));

	    // instanciate the servant for the interation manager
	    InterationManagerImpl servant = 
		new InterationManagerImpl(orb, rootpoa, ie_r);
	    org.omg.CORBA.Object o = servant._this_object(orb);
	
	    // create the Adaptacao context
	    NameComponent [] name = new NameComponent[1];
	    name[0] = new NameComponent("Adaptacao","context");	  

	    boolean contexto_existe = false;
	    NamingContextExt subContext = null;
	    System.out.println("Tentando criar contexto Adaptacao...");
	    try {   
		subContext =
		    NamingContextExtHelper.narrow(rootContext.bind_new_context(name));
	    }
	    catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ab) {
		System.out.println("Contexto Adaptacao ja existe...");
		contexto_existe = true;
	    }
  
	    if (contexto_existe)
		subContext =
		    NamingContextExtHelper.narrow(rootContext.resolve(name));
	    
	    // register the InterationManager object with the name service
	    NameComponent [] sname = new NameComponent[1];
	    sname[0] = new NameComponent("InterationManager","service");
	    subContext.rebind(sname, o);
	    
	    // activates the poaManager
	    rootpoa.the_POAManager().activate();

	    // wait for requests
	    orb.run();
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
}	
