/*
 * RepositoryServer
 * Version 1.0
 */
package EventEvaluator.Interation;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosTransactions.*;
import org.omg.CosNaming.*;

import java.io.*;

/** 
 * Implements the Resource Event Repository Server. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class ServerRepository {

    /**
     * Creates a Resource Event Repository.
     * Creates the <code>Persistence</code> object.
     * Obtains reference to orb and root poa. Creates the resource
     * event poa.  Instantiate the servant that implements the
     * EventEvaluator.Resource.ResourceEventRepository interface and
     * register it with the name server. Instantiate the servant 
     * locator for resource event objects.
     * Waits for users requests.
     * @param args[0] the machine where database <code>adaptacao</code> must reside.
     */
    public static void main(String[] args) {
    
	if (args.length != 1) {
	    System.err.println("Use: jaco -DOAPort=5024 EventEvaluator.Interation.ServerRepository db_machine");
	    System.exit(1);
	}

	// create the persistence object, responsible for keeping in disc
	// the state of ResourceEvent objects
	System.out.println (args[0]);
	Persistence pobj = new Persistence(args[0]);

	// obtain reference to orb
	org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
	try {
	    // obtain reference to root poa
	    POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	    
	    // define policies for a poa responsible for persistent objects
	    // that will use servant manager / locator
	    Policy[] policies = new Policy[5];
	    policies[0] = rootpoa.create_request_processing_policy
		(RequestProcessingPolicyValue.USE_SERVANT_MANAGER);
	    policies[1] = 
		rootpoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT);
	    policies[2] = 
		rootpoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);
	    policies[3] = rootpoa.create_implicit_activation_policy
		(ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
	    policies[4] = rootpoa.create_servant_retention_policy
		(ServantRetentionPolicyValue.NON_RETAIN);
	    
	    // create poa for ResourceEvent objects
	    POA ie_poa = 
		rootpoa.create_POA("InterationEventPOA", rootpoa.the_POAManager(), policies);
	    
	    // obtain reference to name service
	    NamingContextExt rootContext = 
		NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));
	
	    // instanciate and register the servent locator of ResourceEvent objects
	    InterationEventLocatorImpl ie_locator = new InterationEventLocatorImpl(orb, pobj);
	    ie_poa.set_servant_manager(ie_locator._this(orb));

	    // instanciate the servant for the repository of ResourceEvent objects
	    // using implicit activation
	    InterationEventRepositoryImpl repositoryservant = 
		new InterationEventRepositoryImpl(orb, ie_poa, pobj, ie_locator);
	    org.omg.CORBA.Object o = repositoryservant._this_object(orb);

	    // create the Adaptacao context
	    NameComponent [] name = new NameComponent[1];
	    name[0] = new NameComponent("Adaptacao","context");	  

	    boolean contexto_existe = false;
	    NamingContextExt subContext = null;
	    System.out.println("Tentando criar contexto Adaptacao...");
	    try {   
		subContext =
		    NamingContextExtHelper.narrow(rootContext.bind_new_context(name));
	    }
	    catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ab) {
		System.out.println("Contexto Adaptacao ja existe...");
		contexto_existe = true;
	    }
  
	    if (contexto_existe)
		subContext =
		    NamingContextExtHelper.narrow(rootContext.resolve(name));
	    
	    // register the repository of ResourceEvent objects with the name service
	    NameComponent [] sname = new NameComponent[1];
	    sname[0] = new NameComponent("InterationEventRepository","service");
	    subContext.rebind(sname, o);
	    
	    // activates the poaManager
	    rootpoa.the_POAManager().activate();

	    // wait for requests
	    orb.run();
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
}	
