/*
 * ActiveEvent
 * Version 1.0
 */

package EventEvaluator.Resource;

import java.util.*;
import java.sql.*;
import java.io.*;
import java.lang.*;

/**
 * Event that is beeing evaluated by the Resource Manager
 * 
 * @author Francisco Silva
 * @version 1.0
 */
public class ActiveEvent {

    private String eid;
    private ResourceEvent re;
    private String expression;
    private int duration_time;
    private Vector exPid;
    private boolean is_suspended;
    private Utilities.ProcessaExpressao procExp;

    /**
     * Obtains data from resource event <code>re</code>.
     * @param re reference to resource event related to this active event
     */
    public ActiveEvent(ResourceEvent re) {
	this.re = re;
	this.eid = re.eid();
	this.expression = re.expression();
	this.duration_time = re.duration_time();
	this.is_suspended = false;	
	this.procExp = new Utilities.ProcessaExpressao();
	// System.out.println("New Active Event. Expression: "+this.expression);
	// this.exPid = new Vector(1);
	this.exPid = procExp.extrai_parametros(this.expression);
	// System.out.print("Parameters: ");
	/* 
	 * for (Enumeration exPidList = exPid.elements() ; exPidList.hasMoreElements() ;) {
	 *   System.out.print(exPidList.nextElement());
	 * }
	 * System.out.println();
	*/
    }

    /**
     * Returns the list of parameter identifications contained in the boolean expression
     * of thie active event.
     * @return the list of <code>pid</code>
     */
    public Vector exPid () {
	return exPid;
    }

    /**
     * Checks if the boolean expression contains parameter <code>pid</code>.
     * @param pid the parameter identification
     * @return <code>TRUE</code> if parameter <code>pid</code> is found on
     * expression, <code>FALSE</code> otherwise
     */ 
    public boolean containsPid (String pid) {
	return exPid.contains(pid);
    }

    /**
     * Returns the boolean expression of this event.
     * @return a string containing the boolean expresion
     */
    public String get_expression() {
	return expression;
    }

    /**
     * Returns the event identification.
     * @return a string containing the event identification
     */
    public String eid() {
	return eid;
    }

    /**
     * Returns the duration time of this event.
     * @return the duration time (in seconds)
     */
    public int duration_time() {
	return duration_time;
    }

    /**
     * Returns if the evaluation of this active event is supended or not.
     * @return <code>TRUE</code> if the evaluation of this event
     * is suspended, <code>FALSE</code> otherwise
     */
    public boolean is_suspended() {
	return is_suspended;
    }

    /**
     * Suspend the evaluation of this active event.
     */
    public void suspend() {
	is_suspended = true;
    }
  
    /**
     * Resume the evaluation of this active event.
     */  
    public void resume() {
	is_suspended = false;
    }
} // end class ActiveEvent



