/*
 * Notifier
 * Version 1.0
 */

package EventEvaluator.Resource;

import java.util.*;
import java.lang.*;
import java.io.*;

import Utilities.Log;

import org.omg.CosNaming.*;

/**
 * Notifies the occurence of events
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Notifier extends Thread {

    private org.omg.CORBA.ORB orb;
    private NotifierPushSupplier  pushNotifier;
    private Log logObj;

    public Notifier (org.omg.CORBA.ORB porb) {
	this.orb = porb;
	pushNotifier = new NotifierPushSupplier(orb);
	logObj = logObj.Instance();
    }

    /**
     * Notifies the occurence of events
     * @param list the list of events that occured
     */
    public void notify (Vector list) {
	for (Enumeration e = list.elements() ; e.hasMoreElements() ;) {
	    NotifyElement ne = (NotifyElement) e.nextElement();
	    if (ne != null) {
		System.out.println("==> Event "+ne.ae.eid()+" on "+
				   ne.meid+" occured !!! <==");
		pushNotifier.notify(ne.ae.eid()+"."+ne.meid);

		if (logObj.logging()) {
		    String sts = new String("Event "+ne.ae.eid()+" on "+ne.meid+
					    " notified\n\n");
		    logObj.writeStr(sts);
		}
	    }
	}
	list = null;
    }

    /** 
     * Checks if the notifier thread is running
     */
    public void alive () {
	System.out.println ("Notifier is alive");
    }

} // end class Notifier





