/*
 * ParameterValue
 * Version 1.0
 */

package EventEvaluator.Resource;

import java.util.*;
import java.sql.*;
import java.io.*;
import java.lang.*;

/**
 * Current value of a parameter beeing monitored on an entity.
 * 
 * @author Francisco Silva
 * @version 1.0
 */
public class ParameterValue {

    private String pid;
    private int current_range;
    private boolean is_suspended;

    /**
     * Receives the initial value for the parameter beeing monitored on an entity.
     * @param pid parameter identification
     * @param initial_range the initial range of this parameter
     */
    public ParameterValue(String pid, int initial_range) {
	this.current_range = initial_range;
	this.pid = pid;
	this.is_suspended = false;
    }

    /**
     * Updates the current value of the parameter
     * @param new_range the new value of the parameter
     */
    public void set_range (int new_range) {
	current_range = new_range;
    }

    /**
     * Returns the current value of the parameter
     * @return the current value of the parameter
     */
    public int get_range () {
	return current_range;
    }

    /**
     * Returns the parameter identification
     * @return parameter identification
     */
    public String get_pid() {
	return pid;
    }

    /**
     * Check if the monitoring of the paramter is suspended
     * @return <code>TRUE</code> if the monitoring processes of the parameter
     * is suspended, <code>FALSE</code> otherwise
     */
    public boolean is_suspended() {
	return is_suspended;
    }

    /**
     * Inform that the monitoring process of this parameter is suspended.
     * This method is callend whent the RMO responsible for monitoring this
     * parameter on the entity has called the method <code>rmo_suspend</code>
     * informing that it monitoring process is suspended. In this case, the
     * current value the parameter is no longer valid and should not be used.
     */
    public void suspend() {
	is_suspended = true;
    }

    /**
     * Inform that the monitoring process of this parameter was resumed.
     * This method is callend whent the RMO responsible for monitoring this
     * parameter on the entity has called the method <code>rmo_resume</code>
     * informing that it monitoring process was resumed. 
     */  
    public void resume() {
	is_suspended = false;
    }
}

