/*
 * ResourceEventImpl
 * Version 1.0
 */
package EventEvaluator.Resource;

import java.util.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

/** 
 * Implements the EventEvaluator/Resource/ResourceEvent interface.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class ResourceEventImpl extends ResourceEventPOA {
    private String eid;                 // resource event id
    private String strIOR;              // stringfied ior 
    private String description;         // description
    private String expression;
    private int    duration_time;
    private MonitoredEntities.EntityType metype;  // reference to Monitored Entity Type
    org.omg.CORBA.ORB orb;              // orb reference
    Persistence pobj;                   // persistence object

    /**
     * Loads the state of the resource event this object is incarnating
     * from persistent storage.
     * Receives an reference to the <code>Persistence</code> object and calls
     * the method <code>RELoadState</code> that returns the state of the resource
     * event this object is incarnating (identified by <code>eid</code>).
     * <p>
     * The state of the resource event is returned in an object array
     * that must have its elements casted to the specific type of each class variable.
     * @param eid identification of the resource event the object is incarnating
     * @param orb reference to the orb
     * @param pobj reference to the Persistence object
     */
    public ResourceEventImpl(String eid, org.omg.CORBA.ORB orb, Persistence pobj)
    {
	this.eid = eid;
	this.orb = orb;
	this.pobj = pobj;
	Object state[] = new Object[6];
	state = pobj.RELoadState(eid);
	this.eid = (String)state[0];
	this.strIOR = (String)state[1];
	this.description = (String)state[2];
	this.expression = (String)state[3];
	Integer iduration_time = new Integer((String)state[4]);
	this.duration_time = iduration_time.intValue();
	String str_metype = (String)state[5];
	org.omg.CORBA.Object obj = orb.string_to_object(str_metype);
	this.metype = MonitoredEntities.EntityTypeHelper.narrow(obj);
    }

    // the methods eid() and description() are derived from
    // Event interface

    /**
     * Returns the identification of this resource event.
     * @return the identification of this resource event
     */
    public String eid() {
	return this.eid;
    }

    /**
     * Returns the description of this resource event.
     * @return the description of this resource event
     */
    public String description() {
	return this.description;
    }

    // the methods expression(), duration_time() and metype()
    // are derived from ResourceEvent interface

    /**
     * Returns the expression of this resource event.
     * This is a boolean expression that must stay <code>true</code> for 
     * at least <code>duration_time</code> seconds in order to trigger
     * the event.
     * 
     * @return the expression of this resource event
     */
    public String expression() {
	return this.expression;
    }

    /**
     * Returns the time the event expression must stay <code>true</code> in order
     * trigger the event.
     * @return the duration time necessary to trigger the event
     */
    public int duration_time() {
	return this.duration_time;
    }

    /**
     * Returns a reference for the monitored entity type this event relates to.
     * @return reference to the entity type this event relates to
     */
    public MonitoredEntities.EntityType metype() {
	return this.metype;
    }

} // end class ResourceEventImpl






