/*
 * ResourceEventRepositoryImpl
 * Version 1.0
 */
package EventEvaluator.Resource;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;

/** 
 * Implements the EventEvaluator.Resource.ResourceEventRepository interface.
 * 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class ResourceEventRepositoryImpl extends ResourceEventRepositoryPOA {

    private org.omg.CORBA.ORB orb;
    private POA the_poa; 
    private MonitoredEntities.EntityRepository eREP;
    private Persistence pobj;
    private Utilities.Calculadora.Calculadora calc;
    private Utilities.ProcessaExpressao procExp;
    private ResourceEventLocatorImpl re_locator;
    
    /**
     * Receive references to orb, poa and <code>Persistence</code> objects.
     * @param orb reference to the orb
     * @param the_poa refence to the poa
     * @param Pobj reference to Persistence object
     * @param eREP reference to monitored entities repository
     * @param re_locator reference to resource event locator
     */
    public ResourceEventRepositoryImpl(org.omg.CORBA.ORB orb, POA the_poa, 
				       Persistence pobj, 
				       MonitoredEntities.EntityRepository eREP,
				       ResourceEventLocatorImpl re_locator)
    {
	this.orb = orb;
	this.the_poa = the_poa;
	this.pobj = pobj;
	this.eREP = eREP;
	this.re_locator = re_locator;
	calc = new Utilities.Calculadora.Calculadora();
	procExp = new Utilities.ProcessaExpressao();
    }

    /** 
     * Creates a resource event object.
     * Creates a resource event object and saves it state to persistent storage.
     * @param eid the identification of the object being created
     * @param description its description 
     * @param expression its expression 
     * @param duration_time the duration time for the expression given
     * @param metype the monitored entity type related with this resource event
     * @return the ior reference for the resource event object that was created
     */
    public ResourceEvent store_resource_event(String eid, String description, 
					      String expression, int duration_time, 
					      MonitoredEntities.EntityType metype) 
      throws EventEvaluator.EventAlreadyExist,
             EventEvaluator.InvalidExpression, 
             EventEvaluator.ParameterExpressionError {
	if (pobj.REExist(eid))
	  throw new EventEvaluator.EventAlreadyExist(eid); 
	// System.out.println("criando evento de recurso " + eid);
	ResourceEvent re = null;
	try
	    {
	      validateExpression(eid, expression);
	      org.omg.CORBA.Object obj = the_poa.create_reference_with_id
		(eid.getBytes(), ResourceEventHelper.id());
	      re = ResourceEventHelper.narrow(obj);
	    }
	catch (org.omg.PortableServer.POAPackage.WrongPolicy wp)
	    {
		wp.printStackTrace();
	    }
	catch (EventEvaluator.InvalidExpression ie)
	    {
	      throw ie;
	    }
	catch (EventEvaluator.ParameterExpressionError pee)
	    {
	      throw pee;
	    }
	catch (Exception exc) {
	    System.out.println("Exception on method store_resource_event()");
	}
	String strIOR = orb.object_to_string(re);
	String strIOR_metype = orb.object_to_string(metype);

	// save object state
	pobj.RESaveState (eid, strIOR, description, expression, duration_time, strIOR_metype);
	System.out.println("Evento de Recurso " + eid + " criado...");
	// System.out.println(strIOR);
	return re;
    }

    /**
     * Destroy an resource event object by removing it from persistent storage.
     * @param eid the identification of the resource event to be destroyed
     */
    public void erase(String eid) throws EventEvaluator.NoSuchEvent {
	System.out.println("tentando excluir evento de recurso " + eid + "...");
	if (! pobj.REExist(eid))
	    throw new EventEvaluator.NoSuchEvent(eid);
	pobj.REDeleteState (eid);
	re_locator.free(eid);
	System.out.println("Evento de Recurso " + eid + " excluido...");
    }

    /**
     * Returns the ior reference for the resource event identified by <code>eid</code>.
     * @param eid the identification of the resource event
     * @return the reference for the resource event
     */
    public ResourceEvent get_re_ref(String eid) throws EventEvaluator.NoSuchEvent {
	if (! pobj.REExist(eid))
	    throw new EventEvaluator.NoSuchEvent(eid); 
	String strIOR = pobj.REGetRef(eid);
	ResourceEvent re =
	    ResourceEventHelper.narrow(orb.string_to_object(strIOR));
	System.out.println("retornando referencia a " + eid + "...");
	// System.out.println(strIOR);
	return re;
    }

    /**
     * Check if the expression is valid.
     * Verify if the parameters identification (pid's) that the expression
     * contains exist and if the expression is syntatically correct.
     */
    private void validateExpression(String eid, String expression)
      throws EventEvaluator.InvalidExpression, 
             EventEvaluator.ParameterExpressionError, 
	     Exception { 
      Vector parList = procExp.extrai_parametros(expression);
      Vector calcLines = new Vector(1);

      try {
	for (Enumeration e = parList.elements() ; e.hasMoreElements() ;) {
	  String pid = (String) e.nextElement();
	  
	  // check if parameter exist
	  MonitoredEntities.Parameter pref = eREP.get_parameter_ref(pid);
	
	  // mount a line declaring the parameter to the calculator
	  String line = pid + "=1";
	  calcLines.add(line);
	} 

	// the last line to be submitted to the calculator should
	// be the expression
	calcLines.add(expression);
	
	// calls the Calculator
	String calcR;
	for (int i = 0; i < calcLines.size()-1 ; i++) {
	  calcR = calc.calcula((String)calcLines.get(i));
	}
	calcR = calc.calcula((String)calcLines.lastElement());
      }
      catch (MonitoredEntities.NoSuchParameter nsp) {
	  throw new EventEvaluator.ParameterExpressionError(eid, expression, nsp.pid);
      }
      catch (Utilities.Calculadora.CException ce) {
	throw new EventEvaluator.InvalidExpression(eid, expression);
      }
      catch (Exception exc) {
	throw exc;
      }
  }
 
} // end class ResourceEventRepositoryImpl
  





