/*
 * ServerResourceManager
 * Version 1.0
 */
package EventEvaluator.Resource;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosTransactions.*;
import org.omg.CosNaming.*;

import java.io.*;

/** 
 * Implements the server process that contains the resource event evaluator.
 * @author Francisco Silva
 * @version 1.0
 */
public class ServerResourceManager {

    public static Utilities.Log logObj;

    /**
     * Creates the resource event evaluator servant and register it with the
     * name service.
     */
    public static void main(String[] args) {
    
	if (args.length != 0 && args.length != 1) {
	    System.out.println ("Use: jaco EventEvaluator.Resource.ServerResourceManager [-p]");
	    System.exit(1);
	}
	if ((args.length == 1) && (args[0].equals("-p"))) {
	    logObj = logObj.Instance();
	    logObj.createFile(System.getProperty("user.home")+"/temp/tempo/processaMudancaFaixa.txt");	
	    logObj.setApplicationName("ResourceEventEvaluator");
	}

	try {
	    // obtain reference to orb
	    //org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
      java.util.Properties props = new java.util.Properties();
          props.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
          props.put("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");
          props.put("jacorb.poa.monitoring","off");
          props.put("ORBInitRef.NameService","http://easter3/~eider/NS_Ref");
            //props.put("ORBInitRef.NameService","http://localhost/mestrado/NS_Ref");
            props.put("ORBInitRef.TradingService","http://www.x.y.z/~user/TraderRef");


          org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);

	    // obtain reference to root poa
	    POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	    
	    // obtain reference to name service
	    NamingContextExt rootContext = 
		NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));
    
	    // obtain reference to ResourceEvent repository
	    NameComponent [] name_re_r = new NameComponent[2];
	    name_re_r[0] = new NameComponent("Adaptacao","context");
	    name_re_r[1] = new NameComponent("ResourceEventRepository","service");
	    System.out.println ("obtendo referencia ao repositorio de eventos de recurso");   
	    ResourceEventRepository re_r = 
		ResourceEventRepositoryHelper.narrow(rootContext.resolve(name_re_r));

	    // instanciate the servant for the resource manager
	    ResourceManagerImpl servant = 
		new ResourceManagerImpl(orb, rootpoa, re_r);
	    org.omg.CORBA.Object o = servant._this_object(orb);
	
	    // create the Adaptacao context
	    NameComponent [] name = new NameComponent[1];
	    name[0] = new NameComponent("Adaptacao","context");	  

	    boolean contexto_existe = false;
	    NamingContextExt subContext = null;
	    System.out.println("Tentando criar contexto Adaptacao...");
	    try {   
		subContext =
		    NamingContextExtHelper.narrow(rootContext.bind_new_context(name));
	    }
	    catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ab) {
		System.out.println("Contexto Adaptacao ja existe...");
		contexto_existe = true;
	    }
  
	    if (contexto_existe)
		subContext =
		    NamingContextExtHelper.narrow(rootContext.resolve(name));
	    
	    // register the ResourceManager object with the name service
	    NameComponent [] sname = new NameComponent[1];
	    sname[0] = new NameComponent("ResourceManager","service");
	    subContext.rebind(sname, o);
	    
	    // activates the poaManager
	    rootpoa.the_POAManager().activate();

	    // wait for requests
	    orb.run();
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
}	
