/*
 * RepTimer
 * Version 1.0
 */
package EventEvaluator.Resource;

import java.util.*;
import java.io.*;
import java.lang.*;

/** 
 * Implements the timer that periodically checks if there is active
 * events whose occurence must be notified.
 * @author Francisco Silva
 * @version 1.0
 */
public class Timer extends Thread {
    private TimerList timerList;

    /**
     * Receive the list of active events whose expression are currently <code>TRUE</code>.
     * @param timerList reference to the list of active events
     */
    public Timer(TimerList timerList) {
	this.timerList = timerList;
    }

    /**
     * Wake-up every second and calls a method that verifys if there is active
     * events whose occurence must be notified.
     */
    public void run () {
	while (true) {
	    try {
		// sleep one second
		Thread.sleep(1000);
		// calls timerList
		timerList.generateNotifyList();
	    }
	    catch (InterruptedException e) {
		System.err.println(e.toString());
	    }
	}
    }

} // end class RepTimer
