/*
 * CurrentRange
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

/** 
 * Hold the range in which the parameter being monitored is operating.
 * This class holds the current percentage of processor usage calculated by the
 * <code>CpuMonitor</code> object. It also determines the range this percentage
 * corresponds to. Finally, it calls a notifier object when there is a change in
 * the operation range for the parameter being monitored.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class CurrentRange {

    double current_value;
    Parameter par;
    int quantr;
    int current_range;
    Notifier notifierThread;
    range rr[];

    /** 
     * Obtains reference to the parameter being monitored and the notifier object.
     * This constructor receives the references for the parameter being monitored
     * and for the notifier object that will be called whenever the operation range
     * changes. It also display on screen all possible ranges of this parameter.
     * @param par reference for the parameter being monitored
     * @param notifierThread reference for the notifierThread
     */
    public CurrentRange (Parameter par, Notifier notifierThread) {
	this.par = par;
	this.notifierThread = notifierThread;

	// obtaining the ranges of my parameter
	// System.out.println("Faixas de valores deste parametro: ");
	this.quantr = par.num_ranges();
	this.rr = new range[this.quantr];
	this.rr = par.ranges();
	this.current_range = -1;

	// printing the ranges on screen
	System.out.println("Parameter ranges:");
	for (int i=0; i<quantr; i++)
	    System.out.println(Double.toString(rr[i].initial_value)+" - "
			       +Double.toString(rr[i].final_value));
	System.out.println();
    }

    /**
     * Receives the current percentage of processor use and determines the 
     * corresponded range.
     * Receives the current percentage of processor use calculated by the 
     * <code>CpuMonitor</code>
     * object and then determines the range this percentage corresponds to. 
     * If the operation range has changed, it call the method <code>notify</code> of the 
     * <code>notifierThread</code> which will notify this change to the Event Evaluator.
     * @param new_current_value current value of processor use
     */
    public synchronized void update (double new_current_value) {
	current_value = new_current_value;
	System.out.println("current value: " + current_value);
	int new_current_range = -1;
	for (int i=0; i<quantr; i++) {
	    if ((current_value >= rr[i].initial_value) && 
		(current_value <= rr[i].final_value)) {
		new_current_range = i;
		break;
	    }
	}
	if (new_current_range != current_range) {
	    current_range = new_current_range;
	    notifierThread.notify(new_current_range);
	}
    }

    /**
     * Returns the current range in which the parameter being monitored is operating.
     * @return current_range current range of the parameter being monitored
     */ 
    public synchronized int get () {
	return current_range;
    } 

} // end class CurrentRange

