/*
 * Frequency
 * Version 1.0
 */

package Monitor.Resource;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

/** 
 * Hold the value of the frequency on which the monitoring processes must occur.
 * This class simply encapsulate the value of the frequency in seconds that
 * the monitoring process must occur.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Frequency {
    int the_frequency;

    /*
     * Receives the initial frequency for the monitoring process.
     * @param initial_frequency value of the initial frequency to be used
     */
    public Frequency (int initial_frequency) {
	the_frequency = initial_frequency;
    }

    /*
     * Returns the current frequency being used by the monitoring process.
     * @return the frequency being used
     */
    public synchronized int get () {
	return the_frequency;
    }

    /*
     * Changes the frequency in which the monitoring process must be occur.
     * The new frequency to be used by the monitoring process is passed
     * by the user using the method <code>change_frequency</code> of the
     * Omr interface.
     * @param new_frequency the new frequency to be used by the monitoring process
     */
    public synchronized void set (int new_frequency) {
	the_frequency = new_frequency;
    }

} // end class Frequency





