/*
 * Inicio
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosTransactions.*;
import org.omg.CosNaming.*;

import java.io.*;

/** 
 * Perform actions common to any RMO object initialization process.
 * This class encapsulates the code that is common to any RMO object
 * when performing its initialization, such as obtaining reference to ORB
 * and registrating itself with the name service.
 * @author Francisco Silva
 * @version 1.0
 */
public class Inicio {

    protected static org.omg.CORBA.ORB orb;
    protected static POA rootpoa;
    protected static EntityRepository et_r;
    protected static Entity me;
    protected static Parameter par;
    protected static EventEvaluator.Resource.ResourceManager rManager;

    /**
     * Obtains reference to ORB, name service, entity and parameter beein monitored
     * and resource event evaluator.
     * @param args[0] the identification of the entity beein monitored
     * @param args[1] the identification of the parameter beeing monitored
     */
    public static void inicializa(String[] args) {
	// obtain reference to orb
	//orb = org.omg.CORBA.ORB.init(args, null);
      java.util.Properties props = new java.util.Properties();
          props.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
          props.put("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");
          props.put("jacorb.poa.monitoring","off");
            props.put("ORBInitRef.NameService","http://easter3/~eider/NS_Ref");
            //props.put("ORBInitRef.NameService","http://localhost/mestrado/NS_Ref");
            props.put("ORBInitRef.TradingService","http://www.x.y.z/~user/TraderRef");


          orb = org.omg.CORBA.ORB.init(args, props);
	try {
	    
	    // obtain reference to root poa
	    rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	    
	    // get reference to the initial naming context	    
	    NamingContextExt nc = 
		NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService")); 
	    // get reference to entities repository
	    NameComponent [] name = new NameComponent[2];
	    name[0] = new NameComponent("Adaptacao","context");
	    name[1] = new NameComponent("EntityRepository","service");
	    System.out.println ("obtaining reference to entities repository");	    
	    et_r = 
		EntityRepositoryHelper.narrow(nc.resolve(name));
	    System.out.println ("reference to entities repository obtained: OK...");

	    // get reference to my monitored entity
	    me = null;
	    par = null;

	    try {
		if (args.length == 3)
		    me = et_r.get_me_ref(args[1]);
		else
		    me = et_r.get_me_ref(args[0]);
	    }
	    catch (NoSuchEntity ne) {
		if (args.length == 3)
		    System.out.println ("Error: Monitored Entity "+
					args[1]+" does not exist!");
		else
		    System.out.println ("Error: Monitored Entity "+
					args[0]+" does not exist!");
		System.exit(1);
	    }

	    // get reference to my parameter
	    try {
		if (args.length == 3)
		    par = et_r.get_parameter_ref(args[2]);
		else
		    par = et_r.get_parameter_ref(args[1]);
	    }
	    catch (NoSuchParameter np) {
		System.out.println ("Error: Parameter "+args[1]+" does not exist!");
		System.exit(1);
	    }

	    if ((me == null) || (par == null)) {
		System.out.println ("Error Monitored Entity or Parameter is null !");
		System.exit(1);
	    }

	    // get reference to resource event evaluator
	    // Not implemented !!
	    // System.out.println ("obtaining reference to resource event evaluator");	  
	    // System.out.println ("reference to resource event evaluator obtained: OK...");
	    
	    // get reference to resource event evaluator
	    NameComponent [] rmname = new NameComponent[2];
	    rmname[0] = new NameComponent("Adaptacao","context");
	    rmname[1] = new NameComponent("ResourceManager","service");
	    System.out.println ("obtaining reference to resource manager");	    
	    rManager = 
		EventEvaluator.Resource.ResourceManagerHelper.narrow(nc.resolve(rmname));
	    System.out.println ("reference to resource manager obtained: OK...");


	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    } // end inicializa

    /**
     * Register this RMO with CORBA name service.
     * @param args[0] the identification of the entity beein monitored
     * @param args[1] the identification of the parameter beeing monitored
     * @param o reference to ORB
     */
    public static void registra_omr (String args[], org.omg.CORBA.Object o) {
	try {
	    // get reference to the initial naming context	    
	    NamingContextExt nc = 
		NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));

	    // create the Adaptacao/Omr context
	    NameComponent [] omrName = new NameComponent[2];
	    omrName[0] = new NameComponent("Adaptacao","context");	  
	    omrName[1] = new NameComponent("Omr","context");

	    boolean contexto_existe = false;
	    NamingContextExt omrContext = null;
	    System.out.println("Tentando criar contexto Adaptacao/Omr...");
	    try {   
		omrContext =
		    NamingContextExtHelper.narrow(nc.bind_new_context(omrName));
	    }
	    catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ab) {
		System.out.println("Contexto Adaptacao/Omr ja existe...");
		contexto_existe = true;
	    }
  
	    if (contexto_existe)
		omrContext =
		    NamingContextExtHelper.narrow(nc.resolve(omrName));
	    
	    // register the ResourceManager object with the name service
	    NameComponent [] tname = new NameComponent[1];
	    if (args.length == 3)
		tname[0] = new NameComponent(args[1]+"."+args[2],"service");
	    else
		tname[0] = new NameComponent(args[0]+"."+args[1],"service");
	    omrContext.rebind(tname, o);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

    }

} // end class inicio



