/*
 * Memory
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosTransactions.*;
import org.omg.CosNaming.*;

import java.io.*;

/** 
 * Memory is the main class responsible for implementing a Memory resource monitoring 
 * object. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Memory extends Inicio {

    public static Utilities.Log logObj;

    /**
     * Creates a Memory Resource Monitoring Object.
     * @param args[0] id of the monitored entity
     * @param args[1] id of the parameter beeing monitored
     */
    public static void main(String[] args) {
    
	if (args.length != 2 && args.length != 3) {
	    System.out.println ("Use: jaco Memory [-p] monitored_entity_id parameter_id");
	    System.exit(1);
	}
	if ((args.length == 3) && (args[0].equals("-p"))) {
	    logObj = logObj.Instance();
	    logObj.createFile(System.getProperty("user.home")+"/temp/tempo/notificaMudancaFaixa.txt");	
	    logObj.setApplicationName("Memory RMO");
	}

        inicializa(args);

	try {
	    // instanciate the servant for this omr 
	    RmoMemoryImpl servant = 
		new RmoMemoryImpl(orb, rootpoa, me, par, rManager);
	    org.omg.CORBA.Object o = servant._this_object(orb);

	    // register this omr with name service
	    registra_omr(args, o);

	    // saves the reference for this object to file ./omr-memory.ref

            // create the object reference
            org.omg.CORBA.Object obj =
                rootpoa.servant_to_reference( servant );

	    // register this omr with resource manager
	    System.out.println("Registering this OMR with resource manager");
	    // rManager.rmo_register(RmoHelper.narrow(obj), me.meid(), par.pid());
	    rManager.rmo_register(orb.object_to_string(obj), me.meid(), par.pid());
	    System.out.println("OMR registered with resource manager OK...");

            // print stringified object reference to file
	    // System.out.println("Saving omr reference to file omr-memory.ref");
	    // PrintWriter pw =
	    //   new PrintWriter( new FileWriter("omr-memory.ref"));
            // pw.println(orb.object_to_string( obj ));
            // pw.flush();
            // pw.close();

	    // start the monitoring process
	    servant.startMonitoring();

	    // activates the poaManager
	    rootpoa.the_POAManager().activate();
	    
	    // wait for requests
	    orb.run();
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    } // end main
    
} // end class Memory
