/*
 * Notifier
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;
import Utilities.Log;

/**
 * Notifies the event evaluator whenever there is a change in the operation range of
 * the parameter being monitored. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Notifier extends Thread {

    private Entity me;
    private String meid;
    private Parameter par;
    private String pid;
    private EventEvaluator.Resource.ResourceManager rm;
    private Log logObj;

    /**
     * Receives references for the parameter and entity being monitored as well as
     * for the resource event evaluator that must be notified when changes occur.
     * @param me the monitored entity
     * @param par the parameter being monitored
     * @param rm the event evaluator that must be notified
     */ 
    public Notifier (Entity me, Parameter par, 
		     EventEvaluator.Resource.ResourceManager rm) {

	// the constructor must also receive a reference for
	// the resource event evaluator
	this.me = me;
	this.meid = me.meid();
	this.par = par;
	this.pid = par.pid();
	this.rm = rm;
	logObj = logObj.Instance();
    }
    
    /** 
     * Notifies the resource event evaluator when changes occur.
     * Whenever the operation range on which the parameter being monitored changes,
     * this method notifies the resource event evaluator of the new operation
     * range for the parameter being monitored. It receives the new range from
     * the <code>CurrentRange</code> object.
     * @param range the new operation range for the parameter being monitored
     */
    public void notify (int range) {

	// notify the resource event evaluator
	// and print the new range on screen
	System.out.println("------------------------------");
	System.out.println("Nova faixa de operao: "+range);
	System.out.println("------------------------------");
	rm.change_parameter(meid, pid, range);
	if (logObj.logging()) {
	    String sts = new String(pid + " = " + range + "\n\n");
	    logObj.writeStr(sts);
	}
    }

} // end class Notifier





