/*
 * SuspendMonitor
 * Version 1.0
 */
package Monitor.Resource;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

/** 
 * Suspends and resumes the monitoring thread in response to user requests.
 * This class encapsulates a boolean variable called <code>is_suspended</code>
 * that is set to <code>true</code> when the user calls the <code>Suspend</code>
 * method of the Monitor.Resource/Rmo interface and is set to <code>false</code>
 * when the user calls the <code>Resume</code> method. The monitor thread
 * checks the value of this variable using the method <code>is_montor_suspended</code>
 * and calls <code>wait</code> if its value is <code>true</code>.
 * When <code>Resume</code> is called by the user, a <code>notify</code> is
 * issued and the monitored thread is resumed.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class SuspendMonitor {

    boolean is_suspended;

    /**
     * Initialize <code>is_suspended</code> with <code>false</code>.
     */
    public SuspendMonitor () {
	is_suspended = false;
    }

    /**
     * Suspends the monitor if the user called the <code>Suspend</code> method
     * of the Monitor.Resource/Rmo interface.
     * This method is called by the monitoring thread. If <code>is_suspended</code>
     * is <code>true</code> it suspends the monitoring thread using the
     * <code>wait</code> method.
     */
    public synchronized void is_monitor_suspended () {
	try {
	    synchronized(this) {
		while (is_suspended) {
		    wait();
		}
	    }
	}
	catch (InterruptedException e) {
	    System.err.println("Error suspending the monitor !");
	    System.err.println(e.toString());
	    System.exit(1);
	}	
    }

    /**
     * Set <code>is_suspended</code> to <code>true</code>.
     * When the user calls the <code>Suspend</code> method of the
     * Monitor.Resource/Rmo interface, the servant calls this method
     * that sets <code>is_suspended</code> to <code>true</code>.
     */
    public synchronized void suspend_monitor () {
	is_suspended = true;
    }

    /**
     * Resume the monitoring thread.
     * When the user calls the <code>Resume</code> method of the
     * Monitor.Resource/Rmo interface, the servant calls this method
     * that sets <code>is_suspended</code> to <code>false</code> and
     * resumes the monitoring thread using the <code>notify</code> method.
     */
    public synchronized void unsuspend_monitor () {
	is_suspended = false;
	notify();
    }

} // end class SuspendMonitor
