/*
 * EntityImpl
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

/** 
 * Implements the MonitoredEntities/Entity interface.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class EntityImpl extends EntityPOA {
    private String meid;                // monitored entity id
    private String strIOR;              // stringfied ior
    private EntityType metype;          // Entity Type related with this entity
    private String description;         // description
    private String attributelist[];     // list of attributes
    private int numattr;                // number of attributes in attributelist              
    private org.omg.CORBA.ORB orb;      // orb reference
    private Persistence pobj;           // persistence object

    /**
     * Loads the state of the entity this object is incarnating
     * from persistent storage.
     * Receives an reference to the <code>Persistence</code> object and calls
     * the method <code>ELoadState</code> that returns the state of the entity
     * this object is incarnating (identified by <code>meid</code>).
     * <p>
     * The state of the entity is returned in an vector
     * that must have its elements casted to the specific type of each class variable.
     * @param meid identification of the entity the object is incarnating
     * @param orb reference to the orb
     * @param pobj reference to the Persistence object
     */
    public EntityImpl(String meid, org.omg.CORBA.ORB orb, Persistence pobj)
    {
	this.meid = meid;
	this.orb = orb;
	this.pobj = pobj;
	Vector state = new Vector();
	state = pobj.ELoadState(meid);
	// this.meid = (String)state.elementAt(0);
	this.strIOR = (String)state.elementAt(1);
	String metid = (String)state.elementAt(2);
	String etIORstr = pobj.ETGetRef(metid);
	this.metype =
	    EntityTypeHelper.narrow(orb.string_to_object(etIORstr));	
	this.description = (String)state.elementAt(3);
	this.numattr = state.size()-4;
	attributelist = new String[numattr];
	int indice = 4;
	for (int i=0; i < state.size()-4; i++) {
	    this.attributelist[i]=(String)state.elementAt(indice);
	    indice = indice + 1;
	}
    }

    /**
     * Returns the identification of this entity.
     * @return the identification of this entity
     */    
    public String meid() {
	return this.meid;
    }        

    /**
     * Returns the identification of the entity type of this entity.
     * @return the identification of the entity type of this entity
     */    
    public EntityType metype() {
	return this.metype;
    }

    /**
     * Returns the description of this entity.
     * @return the description of this entity
     */    
    public String description() {
	return this.description;
    }

    /**
     * Returns the list of attributes of this entity.
     * @return the list of attributes of this entity
     */            
    public String[] attributes() {
	return this.attributelist;
    }

    /**
     * Returns the number of attributes this entity contains.
     * @return the number of attributes this entity contais
     */    
    public int num_attributes() {
        return this.numattr;
    }       

} // end class EntityTypeImpl

   





