/*
 * EntitytypeImpl
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

/** 
 * Implements the MonitoredEntities/EntityType interface.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class EntityTypeImpl extends EntityTypePOA {
    private String metid;               // monitored entity type id
    private String strIOR;              // stringfied ior 
    private String description;         // description
    private org.omg.CORBA.ORB orb;      // orb reference
    private Persistence pobj;           // persistence object

    /**
     * Loads the state of the entity type this object is incarnating
     * from persistent storage.
     * Receives an reference to the <code>Persistence</code> object and calls
     * the method <code>ETLoadState</code> that returns the state of the entity
     * type this object is incarnating (identified by <code>meid</code>).
     * <p>
     * The state of the entity type is returned in an object array
     * that must have its elements casted to the specific type of each class variable.
     * @param metid identification of the entity type the object is incarnating
     * @param orb reference to orb
     * @param pobj reference to Persistence object
     */
    public EntityTypeImpl(String metid, org.omg.CORBA.ORB orb, Persistence pobj)
    {
	this.metid = metid;
	this.orb = orb;
	this.pobj = pobj;
	Object state[] = new Object[3];
	state = pobj.ETLoadState(metid);
	this.metid = (String)state[0];
	this.strIOR = (String)state[1];
	this.description = (String)state[2];
    }

    /**
     * Returns the identification of this entity type.
     * @return the identification of this entity type
     */    
    public String metid() {
	return this.metid;
    }

    /**
     * Returns the description of this entity type.
     * @return the description of this entity type
     */    
    public String description() {
	return this.description;
    }

} // end class EntityTypeImpl



