/*
 * ParameterImpl
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

/** 
 * Implements the MonitoredEntities/Parameter interface.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class ParameterImpl extends ParameterPOA {
    private String pid;                 // parameter id
    private String strIOR;              // stringfied ior
    private EntityType metype;          // Entity Type related with this parameter
    private String description;         // description
    private range[] ranges;             // parameter ranges
    private int nr;                     // number of parameter ranges
    private org.omg.CORBA.ORB orb;      // orb reference
    private Persistence pobj;           // persistence object

    /**
     * Loads the state of the parameter this object is incarnating
     * from persistent storage.
     * Receives an reference to the <code>Persistence</code> object and calls
     * the method <code>PLoadState</code> that returns the state of the parameter
     * this object is incarnating (identified by <code>pid</code>).
     * <p>
     * The state of the parameter is returned in an vector
     * that must have its elements casted to the specific type of each class variable.
     * @param pid identification of the entity the object is incarnating
     * @param orb reference to orb
     * @param pobj reference to Persistence object
     */
    public ParameterImpl(String pid, org.omg.CORBA.ORB orb, Persistence pobj)
    {
	this.pid = pid;
	this.orb = orb;
	this.pobj = pobj;
	Vector state = new Vector();
	state = pobj.PLoadState(pid);
	// this.pid = (String)state.elementAt(0);
	this.strIOR = (String)state.elementAt(1);
	this.description = (String)state.elementAt(3);
	String metid = (String)state.elementAt(2);
	String etIORstr = pobj.ETGetRef(metid);
	this.metype =
	    EntityTypeHelper.narrow(orb.string_to_object(etIORstr));

	this.nr = (int)(state.size()-4)/2;
	this.ranges = new range[this.nr];
	int indice = 4;
	for (int i=0; i < this.nr; i++) {
	    String strinitial = (String)state.elementAt(indice);
	    String strfinal = (String)state.elementAt(indice+1);
	    indice = indice + 2;
	    Double initialrange = new Double(strinitial);
	    Double finalrange = new Double(strfinal);
	    ranges[i] = new range(initialrange.doubleValue(), finalrange.doubleValue());
	}	
    }

    /**
     * Returns the identification of this parameter.
     * @return the identification of this parameter
     */    
    public String pid() {
	return this.pid;
    }

    /**
     * Returns the description of this parameter.
     * @return the description of this parameter
     */            
    public String description() {
	return this.description;
    }

    /**
     * Returns the identification of the entity type this parameter relates to.
     * @return the identification of the entity type this parameter relates to
     */    
    public EntityType metype() {
	return this.metype;
    }

    /**
     * Returns a list of ranges defined for this parameter.
     * @return a list of ranfes defined for this parameter
     */            
    public range[] ranges() {
	return this.ranges;
    }

    /**
     * Returns the number of ranges defined for this parameter.
     * @return the number of ranfes defined for this parameter
     */   
    public int num_ranges() {
        return nr;
    }


} // end class ParameterImpl

   



