/*
 * Server
 * Version 1.0
 */
package MonitoredEntities;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosTransactions.*;
import org.omg.CosNaming.*;

import java.io.*;

/** 
 * Implements the Entity Repository Server. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Server {

    /**
     * Creates an Entity Repository.
     * @param args[0] the machine where database <code>adaptacao</code> must reside.
     */
    public static void main(String[] args) {
    
	if (args.length != 1) {
	    System.err.println("Use: jaco Server db_machine");
	    System.exit(1);
	}
	// create the persistence object, responsible for keeping in disc
	// the state of EntityType, Entity and Parameter objects
	System.out.println (args[0]);
	Persistence pobj = new Persistence(args[0]);
	
	// obtain reference to orb
java.util.Properties props = new java.util.Properties();
    props.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
    props.put("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");
    props.put("jacorb.poa.monitoring","off");
    props.put("ORBInitRef.NameService","http://easter3/~eider/NS_Ref");
      //props.put("ORBInitRef.NameService","http://localhost/mestrado/NS_Ref");
      props.put("ORBInitRef.TradingService","http://www.x.y.z/~user/TraderRef");


    org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);
      //org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
	try {
	    // obtain reference to root poa
	    POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

	    // define policies for a poa responsible for persistent objects
	    // that will use servant manager / locator
	    Policy[] policies = new Policy[5];
	    policies[0] = rootpoa.create_request_processing_policy
		(RequestProcessingPolicyValue.USE_SERVANT_MANAGER);
	    policies[1] = 
		rootpoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT);
	    policies[2] = 
		rootpoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);
	    policies[3] = rootpoa.create_implicit_activation_policy
		(ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
	    policies[4] = rootpoa.create_servant_retention_policy
		(ServantRetentionPolicyValue.NON_RETAIN);
	    
	    // create poa for entity_type objects
	    POA et_poa = 
		rootpoa.create_POA("EntityTypePOA", rootpoa.the_POAManager(), policies);
	    
	    // create poa for entity objects
	    POA e_poa = 
		rootpoa.create_POA("EntityPOA", rootpoa.the_POAManager(), policies);

	    // create poa for parameter objects
	    POA p_poa = 
		rootpoa.create_POA("ParameterPOA", rootpoa.the_POAManager(), policies);

	    // instanciate and register the servent locator of entity_type
	    EntityTypeLocatorImpl et_locator = new EntityTypeLocatorImpl(orb, pobj);
	    et_poa.set_servant_manager(et_locator._this(orb));

	    // instanciate and register the servent locator of entity
	    EntityLocatorImpl e_locator = new EntityLocatorImpl(orb, pobj);
	    e_poa.set_servant_manager(e_locator._this(orb));

	    // instanciate and register the servent locator of parameter
	    ParameterLocatorImpl p_locator = new ParameterLocatorImpl(orb, pobj);
	    p_poa.set_servant_manager(p_locator._this(orb));	    
	    
	    // instanciate the servant for the repository of entities 
	    // using implicit activation
	    EntityRepositoryImpl repositoryservant = 
		new EntityRepositoryImpl(orb, et_poa, e_poa, p_poa, pobj, et_locator,
					 e_locator, p_locator);
	    org.omg.CORBA.Object o = repositoryservant._this_object(orb);
	    
	    // obtain reference to name service
	    NamingContextExt rootContext = 
		NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));

	    // create the Adaptacao context
	    NameComponent [] name = new NameComponent[1];
	    name[0] = new NameComponent("Adaptacao","context");	  

	    boolean contexto_existe = false;
	    NamingContextExt subContext = null;
	    System.out.println("Tentando criar contexto Adaptacao...");
	    try {   
		subContext =
		    NamingContextExtHelper.narrow(rootContext.bind_new_context(name));
	    }
	    catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound ab) {
		System.out.println("Contexto Adaptacao ja existe...");
		contexto_existe = true;
	    }
  
	    if (contexto_existe)
		subContext =
		    NamingContextExtHelper.narrow(rootContext.resolve(name));
	    
	    // register the repository of entities with the name service
	    NameComponent [] sname = new NameComponent[1];
	    sname[0] = new NameComponent("EntityRepository","service");
	    subContext.rebind(sname, o);
	    
	    // activates the poaManager
	    rootpoa.the_POAManager().activate();
	    
	    // wait for requests
	    orb.run();
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    } // end main
    
} // end class Server


