package Utilities.Calculadora;

import java.util.*;
import java.io.*;

public class Calculadora {
    
  private class Erro {
    String descricao;
    int col;
    
    Erro(String d, int l) {
      descricao = d;
      col = l;
    }

    public String toString() {
      String t = "";
      for (int i = 0; i < col; i++) t += " ";
      return t + ": " + descricao;
    }
  }

  // Tabela de Variaveis 
  private Map Variaveis;

  // Controla se deve-se ou nao compilar a expressao
  private boolean compilando;

  // String com a expressao compilada em notacao pos-fixa
  private String expressaoCompilada;

  // Lista de erros encontrados
  private List erros;

  // O analex que le a expressao atual
  private Yylex a;

  // Token atual
  private Yytoken token;
  
  public Calculadora() {
    apagaVariaveis();
  }

  public void apagaVariaveis() {
    Variaveis = new HashMap();
    Variaveis.put("_", new Double(0.0));
  }
  
  public String calcula(String e) throws Exception {
    reInicializa(e);
    compilando = false;
    double res = expr();
    Variaveis.put("_", new Double(res));
    if (token.tipo != TokenTipo.FIM) 
      erroSint("Fim inesperado: " + TokenTipo.nomes[token.tipo]);
    // imprimeErros(e);
    Double dres = new Double(res);
    Utilities.Calculadora.Resultado result;
    if (!erros.isEmpty())
	throw new CException();
    return dres.toString();
  }

  public String compila(String e) throws Exception {
    reInicializa(e);
    compilando = true;
    expr();
    if (token.tipo != TokenTipo.FIM) 
      erroSint("Fim inespearado: " + TokenTipo.nomes[token.tipo]);
    // imprimeErros(e);
    Utilities.Calculadora.Resultado result;
    if (!erros.isEmpty())
	throw new CException();
    return expressaoCompilada;
  }

  private void reInicializa(String e) throws Exception {
    StringReader exp = new StringReader(e);
    a = new Yylex(exp);
    token = a.yylex();
    expressaoCompilada = "";
    erros = new ArrayList();
  }

  private void erroSint(String msg) {
    erros.add(new Erro(msg, a.yyGetCol()));
  }
  
  private void imprimeErros(String e) {
    if (!erros.isEmpty()) {
      System.out.println(e);
      Iterator it = erros.iterator();
      while (it.hasNext()) {
        Erro er = (Erro) it.next();
        System.out.println(er);
      }
    }
  }

  private double prec4 () throws Exception {
    double f;
    
    switch (token.tipo) {

    case TokenTipo.NUMERO:
      f = Double.valueOf(token.texto).doubleValue();
      if (compilando) expressaoCompilada += f + " ";
      token = a.yylex();
      break;
      
    case TokenTipo.MENOS:
      token = a.yylex();
      f = -prec3();            // Note que assim vale "2+-(3*4)"
      if (compilando) expressaoCompilada += "-unario ";
      break;

    case TokenTipo.NEGACAO:
      token = a.yylex();
      if (prec3() == 0.0) f = 1.0;
      else f = 0.0;
      if (compilando) expressaoCompilada += "! ";
      break;

    case TokenTipo.PAR_ESQ:
      token = a.yylex();
      f = expr();
      if (token.tipo != TokenTipo.PAR_DIR)
        erroSint("Faltando ')', inserido");
      else token = a.yylex();
      break;
      
    case TokenTipo.VARIAVEL:
      String var = token.texto;
      token = a.yylex();

      if (token.tipo == TokenTipo.ATRIBUI) {
        token = a.yylex();
        f = expr();
        Variaveis.put(var, new Double(f));
        if (compilando) expressaoCompilada += "\"" + var + "\" atualiza ";
      }

      else { 
        if (Variaveis.containsKey(var)) {
          f = ((Double) Variaveis.get(var)).doubleValue();
        }
        else {
          erroSint("Variavel usada antes de definicao. Usando 1.0");
          f = 1.0;
        }
        if (compilando) expressaoCompilada += "\"" + var + "\" le ";
      }
      break;
      
    case TokenTipo.FUNCAO:
      String funcao = token.texto;
      token = a.yylex();
      if (token.tipo != TokenTipo.PAR_ESQ)
	erroSint("Faltando '(', inserindo");
      else token = a.yylex();
      f = expr();
      if (token.tipo != TokenTipo.PAR_DIR)
	erroSint("Assumindo a presenca de um ')'");
      else token = a.yylex();
      if (compilando) expressaoCompilada += funcao + " ";
      f = TabelaFuncoes.calcula(funcao, f);
      break;
      
    default:
      // Verifica se e um operador.
      if (token.tipo >= TokenTipo.VEZES && token.tipo <= TokenTipo.NEGACAO) {
        erroSint("O unico operador valido aqui e' '-'");
        token = a.yylex();  
	f = prec3();
      }
      else {
        erroSint("Fator invalido. Trocando por 1.0.");
        f = 1.0;
        token = a.yylex();
      }
      break;
    }
    return f;
  }

  private double prec3() throws Exception {
    double t1, t2;
    int op;

    t1 = prec4();

    while (token.tipo == TokenTipo.VEZES || token.tipo == TokenTipo.DIVIDIDO) {
      op = token.tipo;
      token = a.yylex();
      switch (op) {
      case TokenTipo.VEZES: 
        t1 *= prec3();
        if (compilando) expressaoCompilada += "* ";
        break;
        
      case TokenTipo.DIVIDIDO:
        t2 = prec3();
        if (t2 == 0.0) {
          System.out.print("Divisao por 0.0\nAssumindo resultado == 1.0");
          t1 = 1.0;
        }
        else t1 /= t2;
        if (compilando) expressaoCompilada += "/ ";
        break;
      }
    }
    return t1;
  }
  
  private double prec2() throws Exception {
    double e;
    int op;

    e = prec3();

    while (token.tipo == TokenTipo.MENOS ||
           token.tipo == TokenTipo.MAIS) {
      op = token.tipo;
      token = a.yylex();
      switch (op) {
      case TokenTipo.MENOS:
        e -= prec2();
        if (compilando) expressaoCompilada += "- ";
        break;
      case TokenTipo.MAIS:
        e += prec2();
        if (compilando) expressaoCompilada += "+ ";
        break;
      }
    }
    return e;
  }

  private double prec1() throws Exception {
    double e;
    int op;

    e = prec2();

    while (token.tipo >= TokenTipo.MAIOR && 
           token.tipo <= TokenTipo.DIFERENTE) {
      op = token.tipo;
      token = a.yylex();
      switch (op) {
      case TokenTipo.MAIOR:
        e = (e > prec1()) ? 1.0 : 0.0;
        if (compilando) expressaoCompilada += "> ";
        break;
      case TokenTipo.MENOR:
        e = (e < prec1()) ? 1.0 : 0.0;
        if (compilando) expressaoCompilada += "< ";
        break;
      case TokenTipo.MAIOR_IG:
        e = (e >= prec1()) ? 1.0 : 0.0;
        if (compilando) expressaoCompilada += ">= ";
        break;
      case TokenTipo.MENOR_IG:
        e = (e <= prec1()) ? 1.0: 0.0;
        if (compilando) expressaoCompilada += "<= ";
        break;
      case TokenTipo.IGUAL:
        e = (e == prec1()) ? 1.0 : 0.0;
        if (compilando) expressaoCompilada += "== ";
        break;
      case TokenTipo.DIFERENTE:
        e = (e != prec1()) ? 1.0 : 0.0;
        if (compilando) expressaoCompilada += "!= ";
        break;
      }
    }
    return e;
  }

  private double expr() throws Exception {
    double e, e2;
    int op;

    // Os dois operadores logicos binarios.
    e = prec1();
    while (token.tipo == TokenTipo.ELOGICO ||
           token.tipo == TokenTipo.OULOGICO) {
      op = token.tipo;
      token = a.yylex();
      switch (op) {
      case TokenTipo.ELOGICO:
	e2 = prec1();
	// A prec1 deve ser chamadada antes.
	if (e2 != 0.0 && e != 0.0)
	  e = 1.0;
	else
	  e = 0.0;
        if (compilando) expressaoCompilada += "&& ";
        break;
      case TokenTipo.OULOGICO:
	e2 = prec1();
	// A prec1 deve ser chamadada antes.
	if (e2 != 0 || e != 0.0)
	  e = 1.0;
	else
	  e = 0.0;
        if (compilando) expressaoCompilada += "|| ";
        break;
      }
    }
    return e;
  }

}


	
    
    










