package Teste;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

public class Suspend {

    boolean is_suspended;

    /**
     * Initialize <code>is_suspended</code> with <code>false</code>.
     */
    public Suspend () {
	is_suspended = false;
    }

    /**
     * Suspends the monitor if the user called the <code>Suspend</code> method
     * of the Monitor/Omr/Rmo interface.
     * This method is called by the monitoring thread. If <code>is_suspended</code>
     * is <code>true</code> it suspends the monitoring thread using the
     * <code>wait</code> method.
     */
    public synchronized void is_suspended () {
	try {
	    synchronized(this) {
		while (is_suspended) {
		    wait();
		}
	    }
	}
	catch (InterruptedException e) {
	    System.err.println("Error in Suspend.is_suspended() !");
	    System.err.println(e.toString());
	    System.exit(1);
	}	
    }

    /**
     * Set <code>is_suspended</code> to <code>true</code>.
     * When the user calls the <code>Suspend</code> method of the
     * Monitor/Omr/Rmo interface, the servant calls this method
     * that sets <code>is_suspended</code> to <code>true</code>.
     */
    public synchronized void suspend () {
	is_suspended = true;
    }

    /**
     * Resume the monitoring thread.
     * When the user calls the <code>Resume</code> method of the
     * Monitor/Omr/Rmo interface, the servant calls this method
     * that sets <code>is_suspended</code> to <code>false</code> and
     * resumes the monitoring thread using the <code>notify</code> method.
     */
    public synchronized void resume () {
	is_suspended = false;
	notify();
    }

} // end class Suspend


